window.addEventListener("load", () => {

  let GAME = {
    REMAINING : 20,
    CARDS : [],
    SELECTED : []
  };

  class Card {
    constructor(n, td) {
      this.visible = false;
      this.selected = false;
      Object.defineProperty(this, "number", { value : n, writable : false });
      Object.defineProperty(this, "display", { value : td, writable : false });
    }

    flip() {
      if (this.visible) {
        this.display.innerHTML = "";
      } else {
        this.display.innerHTML = "<img src='" + this.number + ".png' />";
      }
      this.visible = !this.visible;
    }

    isVisible() {
      return this.visible;
    }

    toggle() {
      if (this.selected) {
        this.display.style.background = "";
        this.selected = false;
        let i = GAME.SELECTED.indexOf(this);
        GAME.SELECTED.splice(i,1);
      } else if ( !this.visible && !this.selected && GAME.SELECTED.length < 2) {
        this.display.style.background = "orange";
        this.selected = true;
        GAME.SELECTED.push(this);
      }
    }

  }

  function shuffleArray(tab) {
    for (let i = 0; i < tab.length - 1; i++) {
      let j = Math.trunc(Math.random() * (tab.length - i)) + i;
      let tmp = tab[j];
      tab[j] = tab[i];
      tab[i] = tmp;
    }
  }

  function initArray() {
    let tab = [];
    for (let i = 0; i <= 9; i++) {
      tab.push(i);
      tab.push(i);
    }

    shuffleArray(tab);

    let result = [];
    let k = 0;
    for (let i = 0; i < 4; i++) {
      result[i] = [];
      for (let j = 0; j < 5; j++) {
        let td = document.getElementById("id_" + i + "_" + j);
        let n = tab[k];
        result[i].push(new Card(n, td));
        k++;
      }
    }
    return result;
  }


  let victoryDiv = document.getElementById("id_victory");
  let flipButton = document.getElementById("id_flip");
  let table = document.getElementById("id_table");
  let resetButton = document.getElementById("id_reset");

  function victory() {
    victoryDiv.style.display = "block";
  }
  function reset () {

    victoryDiv.style.display = "";
    GAME.REMAINING = 20;
    GAME.SELECTED = [];
    GAME.CARDS = initArray();
    for (let i = 0; i < 4; i++) {
      for (let j = 0; j < 5; j++) {
        GAME.CARDS[i][j].display.innerHTML = "";
        console.log(i,j, GAME.CARDS[i][j].number);
      }
    }
  }

  reset();

  resetButton.addEventListener("click", reset);

  table.addEventListener("click", (e) => {
    if (e.target.nodeName.toLowerCase() == "td") {
      let id = e.target.id;
      let fields = id.split("_");
      let card = GAME.CARDS[+fields[1]][ +fields[2]];
      card.toggle();
    }
  });

  flipButton.addEventListener("click", () => {
    if (GAME.SELECTED.length != 2) return;

    flipButton.disabled = true;

    GAME.SELECTED.forEach((c) => c.flip());
    if (GAME.SELECTED[0].number == GAME.SELECTED[1].number) {
      GAME.REMAINING -= 2;
      GAME.SELECTED[0].toggle();
      GAME.SELECTED[0].toggle();
      flipButton.disabled = false;
      if (GAME.REMAINING == 0) victory();
    } else {
      setTimeout(() => {
        GAME.SELECTED.forEach((c) => { c.flip(); });
        GAME.SELECTED[0].toggle();
        GAME.SELECTED[0].toggle();
        flipButton.disabled = false;
      }, 5000);
    }
  });


});